# Generated by Django 2.2.13 on 2020-06-12 08:15
from django.db import migrations


class Migration(migrations.Migration):
    # This migration used to do things, but doesn't anymore.
    # We cannot remove it due to keeping migration state consistent

    dependencies = [
        ("emeis_core", "0001_initial"),
    ]

    operations = [
        # migrations.RunPython(migrations.RunPython.noop, migrations.RunPython.noop),
    ]
